@@ -1,13 +1,14 @@
 import asyncio
+import math
+from argparse import ArgumentParser
 
 import aiofiles
-from argparse import ArgumentParser
 from aioconsole import AsynchronousCli
 
 from wizwalker import Wad, utils
-from wizwalker.windows.memory import MemoryHandler
 
 
+# noinspection PyProtectedMember
 class WizWalkerConsole(AsynchronousCli):
     def __init__(self, walker, **kwargs):
         commands = self.get_commands()
@@ -20,10 +21,17 @@ class WizWalkerConsole(AsynchronousCli):
         msg += "send list for the command list"
         return msg
 
+    async def exit_command(self, reader, writer):
+        writer.write("Closing wizwalker, hooks should be rewritten")
+        await self.walker.close()
+        await super().exit_command(reader, writer)
+
     def get_commands(self):
         commands = {}
 
-        attach_parser = ArgumentParser(description="Attach to currently running wiz instances")
+        attach_parser = ArgumentParser(
+            description="Attach to currently running wiz instances"
+        )
         commands["attach"] = (self.attach_command, attach_parser)
 
         login_parser = ArgumentParser("login to a client")
@@ -34,12 +42,15 @@ class WizWalkerConsole(AsynchronousCli):
 
         hook_parser = ArgumentParser(description="Hooks into all processes")
         hook_parser.add_argument("--target_hook", type=str)
+        hook_parser.add_argument("--list", action="store_true", dest="list_hooks")
         commands["hook"] = (self.hook_command, hook_parser)
 
         player_parser = ArgumentParser(description="Output various player information")
         commands["player"] = (self.player_command, player_parser)
 
-        backpack_parser = ArgumentParser(description="Output various backpack information")
+        backpack_parser = ArgumentParser(
+            description="Output various backpack information"
+        )
         commands["backpack"] = (self.backpack_command, backpack_parser)
 
         quest_parser = ArgumentParser(description="Output various quest information")
@@ -71,6 +82,11 @@ class WizWalkerConsole(AsynchronousCli):
         send_parser.add_argument("seconds", type=float)
         commands["send"] = (self.send_command, send_parser)
 
+        getspeed_parser = ArgumentParser(
+            description="get number of units covered in 1 second"
+        )
+        commands["getspeed"] = (self.getspeed_command, getspeed_parser)
+
         teleport_parser = ArgumentParser(description="Teleport to a certain x,y,z")
         teleport_parser.add_argument("x", type=float)
         teleport_parser.add_argument("y", type=float)
@@ -78,6 +94,11 @@ class WizWalkerConsole(AsynchronousCli):
         teleport_parser.add_argument("--yaw", type=float)
         commands["teleport"] = (self.teleport_command, teleport_parser)
 
+        goto_parser = ArgumentParser(description="go to a specific x y")
+        goto_parser.add_argument("x", type=float)
+        goto_parser.add_argument("y", type=float)
+        commands["goto"] = (self.goto_command, goto_parser)
+
         return commands
 
     async def attach_command(self, _, writer):
@@ -94,37 +115,43 @@ class WizWalkerConsole(AsynchronousCli):
         else:
             writer.write(f"client-{client_index}: Logged in\n")
 
-    async def hook_command(self, _, writer, target_hook: str = None):
-        hook = False
-        if target_hook is not None:
-            md_dir = dir(MemoryHandler)
-            try:
-                hook_index = md_dir.index("hook_" + target_hook.replace(" ", "_"))
-            except ValueError:
-                writer.write(f"Hook {target_hook} not found\n")
-                return
-            else:
-                hook = md_dir[hook_index]
+    async def hook_command(
+        self, _, writer, target_hook: str = None, list_hooks: bool = False
+    ):
+        if list_hooks and target_hook:
+            writer.write("You cannot list and hook at the same time smh\n")
+            return
+
+        if not self.walker.clients:
+            writer.write("There are no attached clients to hook to\n")
+            return
+
+        if list_hooks:
+            all_hooks = self.walker.clients[0].get_hooks()
+            writer.write("all hooks:\n" + "\n".join(all_hooks) + "\n")
+            return
 
         for idx, client in enumerate(self.walker.clients):
-            if hook:
-                await getattr(client.memory, hook)()
+            if target_hook:
+                await client.activate_hooks(target_hook.replace(" ", "_"))
                 writer.write(f"client-{idx}: hooked {target_hook}\n")
             else:
-                await client.memory.hook_all()
+                await client.activate_hooks()
                 writer.write(f"client-{idx}: hooked all\n")
 
     async def player_command(self, _, writer):
         for idx, client in enumerate(self.walker.clients):
             writer.write(
-                f"client-{idx}: xyz={await client.xyz()} "
-                f"health={await client.health()} "
-                f"mana={await client.mana()} "
-                f"potions={await client.potions()} "
-                f"gold={await client.gold()} "
-                f"yaw={await client.yaw()} "
-                f"player_base={hex(await client.memory.read_player_base())} "
-                f"player_stat_base={hex(await client.memory.read_player_stat_base())}\n"
+                f"client-{idx}:\n"
+                f"\txyz={await client.xyz()}\n"
+                f"\thealth={await client.health()}\n"
+                f"\tmana={await client.mana()}\n"
+                f"\tpotions={await client.potions()}\n"
+                f"\tgold={await client.gold()}\n"
+                f"\tenergy={await client.energy()}\n"
+                f"\tyaw={await client.yaw()}\n"
+                f"\tplayer_base={hex(await client._memory.read_player_base())}\n"
+                f"\tplayer_stat_base={hex(await client._memory.read_player_stat_base())}\n"
             )
 
     async def backpack_command(self, _, writer):
@@ -132,7 +159,7 @@ class WizWalkerConsole(AsynchronousCli):
             writer.write(
                 f"client-{idx}: used_space={await client.backpack_space_used()} "
                 f"total_space={await client.backpack_space_total()} "
-                f"backpack_struct_addr={hex(await client.memory.read_backpack_stat_base())}\n"
+                f"backpack_struct_addr={hex(await client._memory.read_backpack_stat_base())}\n"
             )
 
     async def quest_command(self, _, writer):
@@ -142,8 +169,8 @@ class WizWalkerConsole(AsynchronousCli):
     async def packet_command(self, _, writer):
         for idx, client in enumerate(self.walker.clients):
             writer.write(
-                f"client-{idx}: socket={await client.memory.read_packet_socket_discriptor()} "
-                f"packet_buffer={await client.memory.read_packet_buffer()}\n"
+                f"client-{idx}: socket={await client._memory.read_packet_socket_discriptor()} "
+                f"packet_buffer={await client._memory.read_packet_buffer()}\n"
             )
 
     async def watch_command(self, _, writer):
@@ -182,17 +209,26 @@ class WizWalkerConsole(AsynchronousCli):
             key = key.upper()
 
         for client in self.walker.clients:
-            asyncio.create_task(client.keyboard.send_key(key, seconds))
+            asyncio.create_task(client._keyboard.send_key(key, seconds))
 
         writer.write("Started\n")
 
+    async def getspeed_command(self, _, writer):
+        client = self.walker.clients[0]
+        start = await client.xyz()
+        await client._keyboard.send_key("W", 1)
+        end = await client.xyz()
+        distance = math.dist((start.x, start.y), (end.x, end.y))
+        writer.write(f"from {start} to {end} covered {distance}\n")
+
     async def teleport_command(self, _, writer, x, y, z=None, yaw=None):
         for client in self.walker.clients:
-            await client.teleport(
-                x=x,
-                y=y,
-                z=z,
-                yaw=yaw
-            )
+            await client.teleport(x=x, y=y, z=z, yaw=yaw)
 
         writer.write("Teleported\n")
+
+    async def goto_command(self, _, writer, x, y):
+        for client in self.walker.clients:
+            asyncio.create_task(client.goto(x, y))
+
+        writer.write("Tasks started\n")
